import numpy as np
import time
import os

MAX_TASKS = 5000
MAX_AGENTS = 5000

def generate_GAP_instance(tasks, agents, filename):
    try:
        with open(filename, "w") as file:
            # Writing the number of tasks and agents to the file
            file.write(f"{tasks} {agents}\n")

            # Writing the capacity of each agent to the file
            capacities = np.random.randint(1, tasks // 2 + 1, size=agents)
            file.write(" ".join(map(str, capacities)) + "\n")

            # Generating random costs for each task and agent combination
            costs = np.random.randint(1, 101, size=(tasks, agents))
            for cost_row in costs:
                file.write(" ".join(map(str, cost_row)) + "\n")

        print("GAP instance generated successfully.")
    except IOError as e:
        print(f"Error opening or writing to file: {e}")

def main():
    while True:
        try:
            tasks = int(input(f"Enter the number of tasks (up to {MAX_TASKS}): "))
            if not (1 <= tasks <= MAX_TASKS):
                raise ValueError(f"Number of tasks must be between 1 and {MAX_TASKS}.")

            agents = int(input(f"Enter the number of agents (up to {MAX_AGENTS}): "))
            if not (1 <= agents <= MAX_AGENTS):
                raise ValueError(f"Number of agents must be between 1 and {MAX_AGENTS}.")
            
            filename = input("Enter the name of the text file to generate: ").strip()
            if not filename:
                raise ValueError("Filename cannot be empty.")

            break
        except ValueError as e:
            print(f"Invalid input: {e}. Please try again.")

    start_time = time.time()

    generate_GAP_instance(tasks, agents, filename)

    end_time = time.time()
    execution_time = end_time - start_time

    print(f"Time taken to generate instance: {execution_time:.2f} seconds.")

if __name__ == "__main__":
    main()
