import numpy as np
import time

def load_GAP_instance(filename):
    try:
        with open(filename, "r") as file:
            # Reading the number of tasks and agents from the file
            tasks, agents = map(int, file.readline().split())

            # Reading the capacities of each agent from the file
            capacities = list(map(int, file.readline().split()))

            # Reading the costs matrix from the file
            costs = []
            for _ in range(tasks):
                costs.append(list(map(int, file.readline().split())))

        return tasks, agents, capacities, costs
    except IOError as e:
        print(f"Error opening or reading file: {e}")
        return None, None, None, None

def greedy_cost_capacity_ratio(tasks, agents, capacities, costs):
    assigned_tasks = [-1] * tasks  # List to store tasks assigned to agents (-1 means unassigned)
    total_cost = 0
    remaining_capacity = capacities.copy()
    
    for task in range(tasks):
        min_ratio = float('inf')
        selected_agent = -1
        for agent in range(agents):
            if remaining_capacity[agent] > 0:  # If agent has capacity
                ratio = costs[task][agent] / (remaining_capacity[agent] + 1e-9)  # Avoid division by zero
                if ratio < min_ratio:
                    min_ratio = ratio
                    selected_agent = agent
        if selected_agent != -1:
            assigned_tasks[task] = selected_agent
            remaining_capacity[selected_agent] -= 1  # Reduce capacity of assigned agent
            total_cost += costs[task][selected_agent]
        else:
            print(f"Task {task + 1} could not be assigned to any agent due to capacity constraints.")

    return assigned_tasks, total_cost

def main():
    filename = input("Enter the filename containing the GAP instance: ").strip()

    # Load GAP instance from file
    tasks, agents, capacities, costs = load_GAP_instance(filename)
    if tasks is None:
        return

    start_time = time.time()

    # Apply greedy cost-capacity ratio heuristic
    assigned_tasks, total_cost = greedy_cost_capacity_ratio(tasks, agents, capacities, costs)

    # Print assigned tasks to agents
    print("Tasks assigned to agents:")
    for agent in range(agents):
        tasks_assigned_to_agent = [str(task + 1) for task, assigned_agent in enumerate(assigned_tasks) if assigned_agent == agent]
        print(f"Agent {agent + 1}: {', '.join(tasks_assigned_to_agent) if tasks_assigned_to_agent else 'None'}")

    end_time = time.time()
    execution_time = end_time - start_time

    print(f"Total cost: {total_cost}")
    print(f"Execution time: {execution_time:.2f} seconds")

if __name__ == "__main__":
    main()
