import numpy as np
import matplotlib.pyplot as plt

# Function to read cities and their coordinates from a text file
def read_cities(filename):
    cities = {}
    with open(filename, 'r') as file:
        for line in file:
            parts = line.split()
            city_name = parts[0]
            x, y = int(parts[1]), int(parts[2])
            cities[city_name] = np.array([x, y])
    return cities

# Function to calculate the distance between two points
def distance(point1, point2):
    return np.sqrt(np.sum((point1 - point2)**2))

# Function to calculate the total distance between centers and cities
def total_distance(centers, cities):
    total_dist = 0
    for city, point in cities.items():
        closest_center = min(centers, key=lambda x: distance(x, point))
        total_dist += distance(closest_center, point)
        plt.plot([point[0], closest_center[0]], [point[1], closest_center[1]], color='gray', linestyle='--', linewidth=0.5)
    return total_dist

# Local search algorithm
def local_search(cities, p):
    # Initialization: we select p random centers
    centers = np.array(list(cities.values()))[np.random.choice(len(cities), p, replace=False)]
    # Local search
    improved = True
    while improved:
        improved = False
        # We iterate over each center
        for i in range(p):
            # We try to move the current center to each point
            for city, point in cities.items():
                # We calculate the total distance with the current center
                current_distance = total_distance(centers, cities)
                # We move the center to the new location
                new_centers = np.copy(centers)
                new_centers[i] = point
                new_distance = total_distance(new_centers, cities)
                # If the distance improves, we update the centers
                if new_distance < current_distance:
                    centers = np.copy(new_centers)
                    improved = True
    return centers, total_distance(centers, cities)

# Read cities from a file
filename = 'cities.txt'
cities = read_cities(filename)

# Example of use
p = 2  # number of centers
best_centers, best_distance = local_search(cities, p)

# Graph cities and centers  
plt.scatter([point[0] for point in cities.values()], [point[1] for point in cities.values()], color='blue', label='Ciudades')
plt.scatter(best_centers[:,0], best_centers[:,1], color='red', marker='x', label='Centros óptimos')
for city, point in cities.items():
    closest_center = min(best_centers, key=lambda x: distance(x, point))
    plt.plot([point[0], closest_center[0]], [point[1], closest_center[1]], color='gray', linestyle='--', linewidth=0.5)
plt.title(f"location of  {p} optimal centers and city allocation")
plt.xlabel('Eje X')
plt.ylabel('Eje Y')
plt.legend()
plt.grid(True)
plt.show()

print("Optimal location of the", p, "centers:")
for i, center in enumerate(best_centers):
    print("Center", i+1, ":", center)
print("total distance:", best_distance)
